/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
 package bufferings.ktr.wjr.client.service;

import java.util.List;

import bufferings.ktr.wjr.shared.model.TestCaseModel;
import bufferings.ktr.wjr.shared.model.TestStore;
import bufferings.ktr.wjr.shared.model.TestStoreConfig;

import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

/**
 * A interface of the KtrWjr GWT-RPC service.
 * 
 * @author bufferings[at]gmail.com
 */
@RemoteServiceRelativePath("ktrwjr.s3gwt")
public interface KtrWjrService extends RemoteService {

  /**
   * Loads the test store configuration.
   * 
   * @return a list of the test store configurations
   */
  public List<TestStoreConfig> loadTestStoreConfigs() throws KtrWjrException;

  /**
   * Loads the test store
   * 
   * @param storeKey
   *          a test store key
   * @return the loaded test store
   */
  public TestStore loadTestStore(String storeKey) throws KtrWjrException;

  /**
   * Runs the test cases
   * 
   * @param testCaseModels
   *          a list of test cases to run
   * @return a list of the test cases with test result
   */
  public List<TestCaseModel> runTests(List<TestCaseModel> testCaseModels)
      throws KtrWjrException;
}
