/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.wjr.client.widget;

import bufferings.ktr.wjr.shared.model.HideSucceededFilter;
import bufferings.ktr.wjr.shared.model.TestCaseModel;
import bufferings.ktr.wjr.shared.model.TestStore;
import bufferings.ktr.wjr.shared.model.TestStoreItem;
import bufferings.ktr.wjr.shared.model.TestSuiteModel;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.DockPanel;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.HorizontalSplitPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * JUnit実行用のコンポジットです。
 * 
 * @author bufferings[at]gmail.com
 */
public class JUnitRunnerComposite extends Composite {

  /** リスナー */
  protected JUnitRunnerCompositeListener listener;

  /** テストストア */
  protected TestStore testStore;

  /** メインパネルを格納するパネル */
  protected SimplePanel containerPanel;

  /** メインパネル */
  protected Widget mainPanel;

  /** 結果 */
  protected ResultComposite resultPanel;

  /** ツリー */
  protected TestsComposite testsPanel;

  /** トレース表示用ラベル */
  protected TraceComposite tracePanel;

  /** 実行(キャンセル)ボタン */
  protected KtrIconButton runTestButton;

  /** 結果クリアボタン */
  protected KtrIconButton clearResultButton;

  protected RunningPopup runningPopup;

  protected boolean isRunning = false;

  protected boolean isCancelRequested = false;

  /** コンストラクタ */
  public JUnitRunnerComposite() {
    containerPanel = new SimplePanel();
    containerPanel.setStyleName("");
    initWidget(containerPanel);
  }

  /**
   * 初期化されているかどうか
   */
  public boolean isInitialized() {
    return mainPanel != null;
  }

  /**
   * コンポジットを初期化します。
   */
  public void initializeComposite() {
    if (isInitialized()) {
      throw new IllegalStateException(
        "Composite.initializeComposite() may only be called once.");
    }

    HorizontalSplitPanel split = new HorizontalSplitPanel();
    split.add(createLeftPanel());
    split.add(createRightPanel());
    split.setSize("100%", "100%");
    split.setStyleName("");

    mainPanel = split;
    containerPanel.setWidget(mainPanel);

    // Popup
    runningPopup = new RunningPopup();

    repaint();
    setRunningState(false);
  }

  protected Widget createLeftPanel() {
    DockPanel panel = new DockPanel();

    resultPanel = new ResultComposite();
    resultPanel.setSize("100%", "60px");

    testsPanel = new TestsComposite();
    testsPanel.setSize("100%", "100%");
    testsPanel.setListener(new TestsCompositeListener() {
      public void onTreeItemSelectionChanged(TestStoreItem selected) {
        if (selected instanceof TestCaseModel) {
          TestCaseModel testCase = (TestCaseModel) selected;
          tracePanel.setTrace(testCase.getTrace());
        } else {
          tracePanel.clear();
        }
      }

      public void onHideSucceededChanged(boolean hide) {
        if (hide) {
          testStore.setFilter(new HideSucceededFilter());
        } else {
          testStore.setFilter(null);
        }
        repaint();
      }

      public void onTreeItemCheckedChanged() {
        updateRunTestButtonDisabled();
      }
    });

    Widget buttonPanel = createButtonPanel();
    buttonPanel.setSize("100%", "30px");

    panel.add(resultPanel, DockPanel.NORTH);
    panel.add(testsPanel, DockPanel.CENTER);
    panel.add(buttonPanel, DockPanel.SOUTH);

    panel.setSize("100%", "100%");
    panel.setCellHeight(testsPanel, "100%");
    panel.setSpacing(2);
    panel.setStyleName("");
    return panel;
  }

  protected Widget createButtonPanel() {
    clearResultButton = new KtrIconButton(new KtrIcon.Arrowrefresh1W(), "Clear All");
    clearResultButton.setTitle("Clear All Results");
    clearResultButton.setSize("100px", "25px");
    clearResultButton.addClickHandler(new ClickHandler() {
      public void onClick(ClickEvent event) {
        testStore.clearResults();
        testStore.applyFilter();
        repaint();
      }
    });

    runTestButton = new KtrIconButton(new KtrIcon.Play(), "Run");
    runTestButton.setSize("100px", "25px");
    runTestButton.addClickHandler(new ClickHandler() {
      public void onClick(ClickEvent event) {
        if (listener != null) {
          if (!isRunning) {
            setRunningState(true);
            listener.onRunTest(testsPanel.getCheckedTestCases());
          } else {
            isCancelRequested = true;
            updateRunTestButtonDisabled();
            listener.onCancelTest();
          }
        }
      }
    });

    HorizontalPanel widget = new HorizontalPanel();
    widget.setHorizontalAlignment(HorizontalPanel.ALIGN_RIGHT);
    widget.add(clearResultButton);
    widget.add(runTestButton);
    widget.setCellHorizontalAlignment(
      clearResultButton,
      HorizontalPanel.ALIGN_LEFT);
    widget.setCellHorizontalAlignment(
      runTestButton,
      HorizontalPanel.ALIGN_RIGHT);
    widget.setStyleName("");
    return widget;
  }

  protected Widget createRightPanel() {
    tracePanel = new TraceComposite();
    tracePanel.setSize("100%", "100%");
    HorizontalPanel panel = new HorizontalPanel();
    panel.add(tracePanel);
    panel.setSize("100%", "100%");
    panel.setSpacing(2);
    panel.setStyleName("");
    return panel;
  }

  /**
   * テストストアを設定します。
   * 
   * @param testStore
   *          テストストア
   */
  public void setTestStore(TestStore testStore) {
    this.testStore = testStore;
  }

  /**
   * リスナーを設定します。
   * 
   * @param listener
   *          リスナー
   */
  public void setListener(JUnitRunnerCompositeListener listener) {
    this.listener = listener;
  }

  public void notifyRunTestsSuccess() {
    repaint();
    setRunningState(false);
  }

  public void notifyRunTestsFailure() {
    repaint();
    setRunningState(false);
  }

  public void notifyRunTestsCancel() {
    repaint();
    setRunningState(false);
  }

  public void notifyRunTestsUpdate() {
    repaint();
  }

  /**
   * 画面を再描画します。
   */
  protected void repaint() {
    TestSuiteModel suite = testStore.getRootSuite();
    resultPanel.setValue(suite.getTotalCount(), suite.getSuccessCount(), suite
      .getFailureCount(), suite.getErrorCount(), suite.getNotYetCount());
    testsPanel.applyTestStore(testStore);
    tracePanel.clear();
    updateRunTestButtonDisabled();
    updateClearResultButtonDisabled();
  }

  protected void setRunningState(boolean state) {
    isRunning = state;
    updateClearResultButtonDisabled();

    if (isRunning) {
      runningPopup.show();
      runTestButton.setKtrIcon(new KtrIcon.Pause());
      runTestButton.setText("Cancel");
      runTestButton.setTitle("Cancel Running");
    } else {
      runTestButton.setKtrIcon(new KtrIcon.Play());
      runTestButton.setText("Run");
      runTestButton.setTitle("Run Checked Tests");
      runningPopup.hide();
      isCancelRequested = false;
      updateRunTestButtonDisabled();
    }
  }

  public void updateRunTestButtonDisabled() {
    if (isCancelRequested || testsPanel.getCheckedTestCases().size() == 0) {
      runTestButton.setDisabled(true);
    } else {
      runTestButton.setDisabled(false);
    }
  }

  public void updateClearResultButtonDisabled() {
    if (isRunning) {
      clearResultButton.setDisabled(true);
    } else {
      TestSuiteModel root = testStore.getRootSuite();
      if (root.getTotalCount() == root.getNotYetCount()) {
        clearResultButton.setDisabled(true);
      } else {
        clearResultButton.setDisabled(false);
      }
    }
  }

  public void showNowLoading() {
    containerPanel.setWidget(createMessagePanel("loading..."));
  }

  public void showCannotLoad() {
    containerPanel.setWidget(createMessagePanel("Cannot load configuration."));
  }

  protected Widget createMessagePanel(String message) {
    Label label = new Label(message);
    label.setStyleName("");

    HorizontalPanel panel = new HorizontalPanel();
    panel.setSize("100%", "100%");
    panel.setHorizontalAlignment(HorizontalPanel.ALIGN_CENTER);
    panel.setVerticalAlignment(HorizontalPanel.ALIGN_MIDDLE);
    panel.add(label);
    panel.setStyleName("");
    return panel;
  }

}
