/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.wjr.client.widget;

import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.InlineLabel;

public class KtrIconLabel extends Composite {

  protected HorizontalPanel mainPanel;

  protected KtrIcon ktrIcon;

  protected InlineLabel label;

  public KtrIconLabel() {
    this(null, null);
  }

  public KtrIconLabel(String text) {
    this(null, text);
  }

  public KtrIconLabel(KtrIcon ktrIcon) {
    this(ktrIcon, null);
  }

  public KtrIconLabel(KtrIcon ktrIcon, String text) {
    mainPanel = new HorizontalPanel();
    mainPanel.setVerticalAlignment(HorizontalPanel.ALIGN_MIDDLE);
    mainPanel.setStyleName("");

    initWidget(mainPanel);
    setKtrIcon(ktrIcon);
    setText(text);
    setHeight("16px");
  }

  public KtrIcon getKtrIcon() {
    return ktrIcon;
  }

  public void setKtrIcon(KtrIcon newIcon) {
    if (ktrIcon != null) {
      mainPanel.remove(ktrIcon);
    }
    ktrIcon = newIcon;
    if (ktrIcon != null) {
      mainPanel.insert(ktrIcon, 0);
    }
  }

  public String getText() {
    return label.getText();
  }

  public void setText(String text) {
    if (text == null || text.length() == 0) {
      if (label != null) {
        mainPanel.remove(label);
      }
    } else {
      if (label == null) {
        label = new InlineLabel();
        mainPanel.add(label);
      }
      label.setText(text);
    }
  }

  public boolean getWordWrap() {
    return label.getWordWrap();
  }

  public void setWordWrap(boolean wrap) {
    label.setWordWrap(wrap);
  }
}
