/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.wjr.client.widget;

import static bufferings.ktr.wjr.shared.util.AssertionUtil.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import bufferings.ktr.wjr.client.KtrWjrPresenter;
import bufferings.ktr.wjr.client.KtrWjrView;
import bufferings.ktr.wjr.shared.model.TestCaseModel;
import bufferings.ktr.wjr.shared.model.TestStore;
import bufferings.ktr.wjr.shared.model.TestStoreConfig;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Panel;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * KtrWjrのビューです。
 * 
 * @author bufferings[at]gmail.com
 */
public class KtrWjrViewImpl implements KtrWjrView {

  protected static final String KOTORI_HOME =
    "<a href='http://code.google.com/p/kotori/' target='_blank' style='font-size: 80%; padding-right: 3px;'>Kotori Web JUnit Runner (Google Code)</a>";

  /** プレゼンター */
  protected KtrWjrPresenter presenter;

  /** 親パネル */
  protected Panel parentPanel;

  /** メインパネル */
  protected SimplePanel mainPanel;

  /** JUnitRunnerComposite */
  protected Map<String, JUnitRunnerComposite> junitRunnerComposites =
    new HashMap<String, JUnitRunnerComposite>();

  /**
   * {@inheritDoc}
   */
  public void initializeView(KtrWjrPresenter presenter, Panel parentPanel) {
    checkNotNull(presenter);
    checkNotNull(parentPanel);
    if (mainPanel != null) {
      throw new IllegalStateException(
        "KtrWjrViewImpl.initializeView() may only be called once.");
    }

    this.presenter = presenter;
    this.parentPanel = parentPanel;

    mainPanel = new SimplePanel();
    mainPanel.setStyleName("");

    VerticalPanel verticalPanel = new VerticalPanel();
    verticalPanel.add(mainPanel);
    verticalPanel.setSize("100%", "100%");
    verticalPanel.setCellHeight(mainPanel, "100%");
    verticalPanel.setStyleName("ui-widget");
    this.parentPanel.add(verticalPanel);

    mainPanel.setWidget(createMessagePanel("loading..."));
    mainPanel.setSize("100%", "100%");

    this.presenter.loadTestStoreConfigs();
  }

  protected Widget createMessagePanel(String message) {
    Label label = new Label(message);
    HorizontalPanel panel = new HorizontalPanel();
    panel.add(label);
    panel.setSize("100%", "100%");
    panel.setCellHorizontalAlignment(label, HorizontalPanel.ALIGN_CENTER);
    panel.setCellVerticalAlignment(label, HorizontalPanel.ALIGN_MIDDLE);
    panel.setStyleName("");
    return panel;
  }

  protected JUnitRunnerComposite createJUnitRunnerComposite(
      final String storeName) {
    final JUnitRunnerComposite widget = new JUnitRunnerComposite();
    widget.setListener(new JUnitRunnerCompositeListener() {

      public void onCancelTest() {
        GWT.log("onCancelTest:" + storeName, null);
        presenter.cancelTests(storeName);
      }

      public void onRunTest(List<TestCaseModel> testCases) {
        GWT.log("onRunTest:" + storeName, null);
        presenter.runTests(storeName, testCases);
      }
    });
    return widget;
  }

  public void onLoadTestStoreFailure(String storeName) {
    junitRunnerComposites.get(storeName).showCannotLoad();
  }

  public void onLoadTestStoreSuccess(String storeName) {
    TestStore testStore = presenter.getTestStore(storeName);
    JUnitRunnerComposite comp = junitRunnerComposites.get(storeName);
    comp.setTestStore(testStore);
    comp.initializeComposite();
  }

  public void onRunTestsCancel(String storeName) {
    junitRunnerComposites.get(storeName).notifyRunTestsCancel();
  }

  public void onRunTestsFailure(String storeName) {
    junitRunnerComposites.get(storeName).notifyRunTestsFailure();
  }

  public void onRunTestsSuccess(String storeName) {
    junitRunnerComposites.get(storeName).notifyRunTestsSuccess();
  }

  public void onRunTestsUpdate(String storeName) {
    junitRunnerComposites.get(storeName).notifyRunTestsUpdate();
  }

  public void onLoadTestStoreConfigsFailure() {
    mainPanel.setWidget(createMessagePanel("Cannot load configurations."));
  }

  public void onLoadTestStoreConfigsSuccess() {
    final Map<String, TestStoreConfig> configs =
      presenter.getTestStoreConfigs();

    final String key = configs.keySet().toArray(new String[0])[0];
    TestStoreConfig value = configs.get(key);

    JUnitRunnerComposite comp = createJUnitRunnerComposite(key);
    junitRunnerComposites.put(key, comp);
    comp.setSize("100%", "100%");
    comp.setStyleName("ui-widget-content");

    HTML titleLabel = new HTML(KOTORI_HOME);
    titleLabel.setStyleName("");

    HorizontalPanel header = new HorizontalPanel();
    header.add(new KtrIconLabel(new KtrIcon.Arrowreturnthick1E(), value
      .getDescription()));
    header.add(titleLabel);
    header.setStyleName("ui-widget-header");
    header.setWidth("100%");
    header.setCellHorizontalAlignment(titleLabel, HorizontalPanel.ALIGN_RIGHT);
    header.setCellVerticalAlignment(titleLabel, HorizontalPanel.ALIGN_BOTTOM);
    header.setSpacing(1);

    VerticalPanel vp = new VerticalPanel();
    vp.add(header);
    vp.add(comp);
    vp.setCellHeight(comp, "100%");
    vp.setSize("100%", "100%");
    vp.setStyleName("");
    mainPanel.setWidget(vp);

    if (presenter.getTestStore(key) == null) {
      junitRunnerComposites.get(key).showNowLoading();
      presenter.loadTestStore(key);
    }
  }

  // public void onLoadTestStoreConfigsSuccess() {
  // final Map<String, TestStoreConfig> configs =
  // presenter.getTestStoreConfigs();
  // final String[] storeKeys = configs.keySet().toArray(new String[0]);
  // final TabPanel tabPanel = new TabPanel();
  //
  // for (Entry<String, TestStoreConfig> entry : configs.entrySet()) {
  // String key = entry.getKey();
  // TestStoreConfig value = entry.getValue();
  //
  // JUnitRunnerComposite comp = createJUnitRunnerComposite(key);
  // junitRunnerComposites.put(key, comp);
  // comp.setSize("100%", "100%");
  //
  // VerticalPanel vp = new VerticalPanel();
  // KtrIconLabel label = new KtrIconLabel(value.getDescription());
  // vp.add(label);
  // vp.add(comp);
  // vp.setCellHeight(comp, "100%");
  // vp.setSize("100%", "100%");
  // vp.setStyleName("");
  // tabPanel.add(vp, value.getStoreName());
  // }
  //
  // tabPanel.addBeforeSelectionHandler(new BeforeSelectionHandler<Integer>()
  // {
  // public void onBeforeSelection(BeforeSelectionEvent<Integer> event) {
  // String storeKey = storeKeys[event.getItem()];
  // if (presenter.getTestStore(storeKey) == null) {
  // junitRunnerComposites.get(storeKey).showNowLoading();
  // presenter.loadTestStore(storeKey);
  // }
  // }
  // });
  //
  // mainPanel.setWidget(tabPanel);
  //
  // tabPanel.selectTab(0);
  //
  // tabPanel.setSize("100%", "100%");
  // tabPanel.getDeckPanel().setSize("100%", "100%");
  // tabPanel.setStyleName("ui-tabs");
  // tabPanel
  // .getTabBar()
  // .setStyleName(
  // "ui-tabs-nav ui-helper-reset ui-helper-clearfix ui-widget-header ui-corner-all");
  // for (int i = 0; i < tabPanel.getTabBar().getTabCount(); i++) {
  // if (i == tabPanel.getTabBar().getSelectedTab()) {
  // ((Widget) tabPanel.getTabBar().getTab(i))
  // .setStyleName("ui-state-default ui-corner-top ui-tabs-selected ui-state-active");
  // } else {
  // ((Widget) tabPanel.getTabBar().getTab(i))
  // .setStyleName("ui-state-default ui-corner-top");
  // }
  // }
  // }
}
