/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.wjr.client.widget;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import bufferings.ktr.wjr.shared.model.TestCaseModel;
import bufferings.ktr.wjr.shared.model.TestStore;
import bufferings.ktr.wjr.shared.model.TestStoreItem;
import bufferings.ktr.wjr.shared.model.TestSuiteModel;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.logical.shared.SelectionEvent;
import com.google.gwt.event.logical.shared.SelectionHandler;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.Tree;
import com.google.gwt.user.client.ui.TreeItem;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

public class TestsComposite extends Composite {

  protected TestsCompositeListener listener;

  private Tree testsTree;

  public TestsComposite() {
    Widget header = createHeader();
    Widget content = createContent();

    VerticalPanel widget = new VerticalPanel();
    widget.add(header);
    widget.add(content);
    widget.setCellHeight(content, "100%");
    widget.setStyleName("");

    initWidget(widget);
  }

  protected Widget createHeader() {
    KtrIconButton expandAll = new KtrIconButton(new KtrIcon.CirclesmallPlus());
    expandAll.setTitle("Expand All");
    expandAll.addClickHandler(new ClickHandler() {
      public void onClick(ClickEvent event) {
        for (int i = 0; i < testsTree.getItemCount(); i++) {
          expand(testsTree.getItem(i));
        }
      }

      protected void expand(TreeItem item) {
        for (int i = 0; i < item.getChildCount(); i++) {
          expand(item.getChild(i));
        }
        item.setState(true);
      }
    });

    KtrIconButton collapseAll =
      new KtrIconButton(new KtrIcon.CirclesmallMinus());
    collapseAll.setTitle("Collapse All");
    collapseAll.addClickHandler(new ClickHandler() {
      public void onClick(ClickEvent event) {
        for (int i = 0; i < testsTree.getItemCount(); i++) {
          collapse(testsTree.getItem(i));
        }
      }

      protected void collapse(TreeItem item) {
        for (int i = 0; i < item.getChildCount(); i++) {
          collapse(item.getChild(i));
        }
        item.setState(false);
      }
    });

    KtrIconButton checkAll = new KtrIconButton(new KtrIcon.Bullet());
    checkAll.setTitle("Check All");
    checkAll.addClickHandler(new ClickHandler() {
      public void onClick(ClickEvent event) {
        for (int i = 0; i < testsTree.getItemCount(); i++) {
          check(testsTree.getItem(i));
        }
        listener.onTreeItemCheckedChanged();
      }

      protected void check(TreeItem item) {
        for (int i = 0; i < item.getChildCount(); i++) {
          check(item.getChild(i));
        }

        TestsTreeItem testsTreeItem = (TestsTreeItem) item;
        testsTreeItem.setChecked(true);
      }
    });

    KtrIconButton uncheckAll = new KtrIconButton(new KtrIcon.RadioOn());
    uncheckAll.setTitle("Uncheck All");
    uncheckAll.addClickHandler(new ClickHandler() {
      public void onClick(ClickEvent event) {
        for (int i = 0; i < testsTree.getItemCount(); i++) {
          uncheck(testsTree.getItem(i));
        }
        listener.onTreeItemCheckedChanged();
      }

      protected void uncheck(TreeItem item) {
        for (int i = 0; i < item.getChildCount(); i++) {
          uncheck(item.getChild(i));
        }

        TestsTreeItem testsTreeItem = (TestsTreeItem) item;
        testsTreeItem.setChecked(false);
      }
    });

    KtrIconButton hideSucceeded = new KtrIconButton(new KtrIcon.Star());
    hideSucceeded.setTitle("Hide Succeeded");
    hideSucceeded.addClickHandler(new ClickHandler() {
      protected boolean hide = false;

      public void onClick(ClickEvent event) {
        hide = !hide;
        KtrIconButton hideSucceeded = (KtrIconButton) event.getSource();
        setStyleName(hideSucceeded.getElement(), "ui-state-active", hide);
        if (listener != null) {
          listener.onHideSucceededChanged(hide);
        }
      }
    });

    HorizontalPanel buttonPanel = new HorizontalPanel();
    buttonPanel.add(expandAll);
    buttonPanel.add(collapseAll);
    buttonPanel.add(checkAll);
    buttonPanel.add(uncheckAll);
    buttonPanel.add(hideSucceeded);
    buttonPanel.setSpacing(1);

    HorizontalPanel header = new HorizontalPanel();
    KtrIconLabel iconLabel = new KtrIconLabel(new KtrIcon.Script(), "Tests");
    header.add(iconLabel);
    header.add(buttonPanel);
    header.setCellVerticalAlignment(iconLabel, HorizontalPanel.ALIGN_MIDDLE);
    header.setCellHorizontalAlignment(iconLabel, HorizontalPanel.ALIGN_LEFT);
    header.setCellHorizontalAlignment(buttonPanel, HorizontalPanel.ALIGN_RIGHT);
    header.setStyleName("ui-widget-header");
    header.setWidth("100%");
    return header;
  }

  protected Widget createContent() {
    TestsTree tree = new TestsTree();
    tree.setStyleName("ktr-tests-tree");
    tree.addSelectionHandler(new SelectionHandler<TreeItem>() {
      public void onSelection(SelectionEvent<TreeItem> event) {
        if (listener != null) {
          listener.onTreeItemSelectionChanged((TestStoreItem) event
            .getSelectedItem()
            .getUserObject());
        }
      }
    });
    tree.setTestsTreeListener(new TestsTreeListener() {
      public void onTreeItemCheckedChanged() {
        listener.onTreeItemCheckedChanged();
      }
    });
    testsTree = tree;

    ScrollPanel content = new ScrollPanel(testsTree);
    content.setSize("100%", "100%");
    content.setStyleName("ui-widget-content");
    return content;
  }

  public void setListener(TestsCompositeListener listner) {
    this.listener = listner;
  }

  public void applyTestStore(TestStore store) {
    // newStoreItemsの取得
    String rootKey = store.getRootSuite().getKey();
    List<TestStoreItem> newStoreItems = store.getSuiteChildren(rootKey);

    // newTreeItemsの取得
    List<TestsTreeItem> newTreeItems = new ArrayList<TestsTreeItem>();
    for (TestStoreItem newStoreItem : newStoreItems) {
      boolean exists = false;
      for (int i = 0; i < testsTree.getItemCount(); i++) {
        TestsTreeItem treeItem = (TestsTreeItem) testsTree.getItem(i);
        String key = ((TestStoreItem) treeItem.getUserObject()).getKey();
        if (key.equals(newStoreItem.getKey())) {
          treeItem.setTestStoreItem(newStoreItem);
          newTreeItems.add(treeItem);
          exists = true;
          break;
        }
      }
      if (!exists) {
        newTreeItems.add(new TestsTreeItem(newStoreItem));
      }
    }

    // 付け替えと子の更新
    testsTree.removeItems();
    for (TestsTreeItem item : newTreeItems) {
      testsTree.addItem(item);
      applyTreeChildren(store, item);
    }

    // 空のsuiteを削除
    for (int i = testsTree.getItemCount() - 1; i >= 0; i--) {
      removeEmptySuite((TestsTreeItem) testsTree.getItem(i));
    }
  }

  protected void applyTreeChildren(TestStore store, TestsTreeItem parentTreeItem) {

    if (parentTreeItem.getUserObject() instanceof TestCaseModel) {
      parentTreeItem.removeItems();
      return;
    }

    // newStoreItemsの取得
    TestSuiteModel parentSuite =
      (TestSuiteModel) parentTreeItem.getUserObject();
    List<TestStoreItem> newStoreItems =
      store.getSuiteChildren(parentSuite.getKey());

    // newTreeItemsの取得
    List<TestsTreeItem> newTreeItems = new ArrayList<TestsTreeItem>();
    for (TestStoreItem newStoreItem : newStoreItems) {
      boolean exists = false;
      for (int i = 0; i < parentTreeItem.getChildCount(); i++) {
        TestsTreeItem treeItem = (TestsTreeItem) parentTreeItem.getChild(i);
        String key = ((TestStoreItem) treeItem.getUserObject()).getKey();
        if (key.equals(newStoreItem.getKey())) {
          treeItem.setTestStoreItem(newStoreItem);
          newTreeItems.add(treeItem);
          exists = true;
          break;
        }
      }
      if (!exists) {
        newTreeItems.add(new TestsTreeItem(newStoreItem));
      }
    }

    // 付け替えと再帰
    parentTreeItem.removeItems();
    for (TestsTreeItem item : newTreeItems) {
      parentTreeItem.addItem(item);
      applyTreeChildren(store, item);
    }
    
    // 全ての子にチェックが入ってるならチェックそうでなければアンチェック
    boolean checked = true;
    for (TestsTreeItem item : newTreeItems) {
      if(!item.isChecked()){
        checked = false;
        break;
      }
    }
    parentTreeItem.setChecked(checked);
  }

  protected void removeEmptySuite(TestsTreeItem item) {
    for (int i = item.getChildCount() - 1; i >= 0; i--) {
      removeEmptySuite((TestsTreeItem) item.getChild(i));
    }

    if (item.getUserObject() instanceof TestSuiteModel
      && item.getChildCount() == 0) {
      item.remove();
    }
  }

  /**
   * チェックされているテストケースを取得します。
   * <p>
   * 一つもチェックされていない場合は空のリストを返します。
   * </p>
   * 
   * @return チェックされているテストケースのリスト
   */
  public List<TestCaseModel> getCheckedTestCases() {
    List<TestCaseModel> ret = new ArrayList<TestCaseModel>();

    Iterator<TreeItem> iterator = testsTree.treeItemIterator();
    while (iterator.hasNext()) {
      TestsTreeItem treeItem = (TestsTreeItem) iterator.next();
      if (treeItem.isChecked()) {
        Object userObject = treeItem.getUserObject();
        if (userObject instanceof TestCaseModel) {
          ret.add((TestCaseModel) userObject);
        }
      }
    }
    return ret;
  }

}
