/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.wjr.client.widget;

import bufferings.ktr.wjr.shared.model.TestCaseModel;
import bufferings.ktr.wjr.shared.model.TestStoreItem;
import bufferings.ktr.wjr.shared.model.TestSuiteModel;
import bufferings.ktr.wjr.shared.model.TestStoreItem.State;

import com.google.gwt.event.logical.shared.ValueChangeEvent;
import com.google.gwt.event.logical.shared.ValueChangeHandler;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.CheckBox;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.TreeItem;

public class TestsTreeItem extends TreeItem {
  protected HorizontalPanel mainPanel;
  protected Label label;
  protected CheckBox check;

  public TestsTreeItem(TestStoreItem testStoreItem) {
    super();
    init();
    setTestStoreItem(testStoreItem);
  }

  protected void init() {
    KtrIcon icon = new KtrIcon.Empty();

    check = new CheckBox();
    check.setStyleName("");
    check.addValueChangeHandler(new ValueChangeHandler<Boolean>() {
      public void onValueChange(ValueChangeEvent<Boolean> event) {
        boolean value = event.getValue();
        TestsTreeItem treeItem = TestsTreeItem.this;
        setValueToChildren(treeItem, value);
        setValueToParent(treeItem);
        ((TestsTree) treeItem.getTree()).onTreeItemCheckedChanged();
      }

      protected void setValueToChildren(TestsTreeItem treeItem, boolean value) {
        for (int i = 0; i < treeItem.getChildCount(); i++) {
          TestsTreeItem child = (TestsTreeItem) treeItem.getChild(i);
          child.setChecked(value);
          setValueToChildren(child, value);
        }
      }

      protected void setValueToParent(TestsTreeItem treeItem) {
        TestsTreeItem parent = (TestsTreeItem) treeItem.getParentItem();
        if (parent != null) {
          boolean parentCheck = true;
          for (int i = 0; i < parent.getChildCount(); i++) {
            TestsTreeItem sibling = (TestsTreeItem) parent.getChild(i);
            if (!sibling.isChecked()) {
              parentCheck = false;
              break;
            }
          }
          parent.setChecked(parentCheck);
          setValueToParent(parent);
        }
      }
    });

    label = new Label();
    label.setWordWrap(false);
    DOM.setStyleAttribute(label.getElement(), "padding", "0 3px 0 3px");
    label.setStyleName("");

    mainPanel = new HorizontalPanel() {
      public void onBrowserEvent(Event event) {
        super.onBrowserEvent(event);
        switch (DOM.eventGetType(event)) {
        case Event.ONMOUSEOVER:
          addStyleName("ui-state-hover");
          break;
        case Event.ONMOUSEOUT:
          removeStyleName("ui-state-hover");
          break;
        }
      }
    };
    mainPanel.setVerticalAlignment(HorizontalPanel.ALIGN_MIDDLE);
    mainPanel.add(check);
    mainPanel.add(icon);
    mainPanel.add(label);
    mainPanel.setStyleName("ktr-tests-tree-item");
    mainPanel.sinkEvents(Event.ONMOUSEOVER | Event.ONMOUSEOUT);

    setWidget(mainPanel);
  }

  public void setTestStoreItem(TestStoreItem testStoreItem) {
    KtrIcon icon;
    if (testStoreItem.getState() == State.SUCCESS) {
      icon = new KtrIcon.Check();
    } else if (testStoreItem.getState() == State.FAILURE) {
      icon = new KtrIcon.Notice();
    } else if (testStoreItem.getState() == State.ERROR) {
      icon = new KtrIcon.Close();
    } else if (testStoreItem.getState() == State.NOT_YET) {
      icon = new KtrIcon.Minus();
    } else if (testStoreItem.getState() == State.SUITE_HAS_NOT_YET) {
      icon = new KtrIcon.Minus();
    } else {
      throw new AssertionError();
    }
    mainPanel.remove(1);
    mainPanel.insert(icon, 1);

    if (testStoreItem instanceof TestSuiteModel) {
      label.setText(testStoreItem.getKey());
      mainPanel.setTitle(testStoreItem.getKey());
    } else {
      TestCaseModel testCaseModel = (TestCaseModel) testStoreItem;

      StringBuilder sb = new StringBuilder(testCaseModel.getName());
      String time = testCaseModel.getTime();
      if (time != null) {
        sb.append(" (").append(time).append("s)");
      }

      label.setText(sb.toString());
      mainPanel.setTitle(sb.toString());
    }

    setUserObject(testStoreItem);
    setSelectedStyle(this.isSelected());
  }

  @Override
  public void setSelected(boolean selected) {
    boolean flg = (this.isSelected() == selected);
    super.setSelected(selected);
    if (!flg) {
      setSelectedStyle(selected);
    }
  }

  protected void setSelectedStyle(boolean selected) {
    mainPanel.setStyleName("ktr-tests-tree-item");
    if (!selected) {
      return;
    }

    TestStoreItem testStoreItem = (TestStoreItem) getUserObject();
    switch (testStoreItem.getState()) {
    case SUCCESS:
      mainPanel.addStyleName("ui-state-highlight");
      break;
    case FAILURE:
      mainPanel.addStyleName("ui-state-error");
      break;
    case ERROR:
      mainPanel.addStyleName("ui-state-error");
      break;
    case NOT_YET:
      mainPanel.addStyleName("ui-state-highlight");
      break;
    case SUITE_HAS_NOT_YET:
      mainPanel.addStyleName("ui-state-highlight");
      break;
    default:
      throw new AssertionError("Unknown State");
    }
  }

  public boolean isChecked() {
    return check.getValue();
  }

  public void setChecked(boolean checked) {
    check.setValue(checked);
  }

}
