/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
 package bufferings.ktr.wjr.client.widget;

import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.KtrNoLeafImageTree;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.Tree;
import com.google.gwt.user.client.ui.TreeItem;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

public class TraceComposite extends Composite {
  protected Tree traceTree;

  public TraceComposite() {
    Widget header = createHeader();
    Widget content = createContent();

    VerticalPanel widget = new VerticalPanel();
    widget.add(header);
    widget.add(content);
    widget.setCellHeight(content, "100%");
    widget.setStyleName("");

    initWidget(widget);
  }

  protected Widget createHeader() {
    HorizontalPanel header = new HorizontalPanel();
    header.add(new KtrIconLabel(new KtrIcon.Pencil(), "Trace"));
    header.setStyleName("ui-widget-header");
    header.setWidth("100%");
    header.setSpacing(1);
    return header;
  }

  protected Widget createContent() {
    traceTree = new KtrNoLeafImageTree();
    traceTree.setStyleName("");

    ScrollPanel content = new ScrollPanel(traceTree);
    content.setSize("100%", "100%");
    content.setStyleName("ui-widget-content");
    return content;
  }

  public void setTrace(String trace) {
    clear();
    if (trace != null && !trace.isEmpty()) {
      String[] splits = trace.split("\n");
      for (String item : splits) {
        TreeItem treeItem = createTreeItem(item);
        traceTree.addItem(treeItem);
      }
    }
  }

  public void clear() {
    traceTree.clear();
  }

  protected TreeItem createTreeItem(String value){
    Label label = new Label(value){
      public void onBrowserEvent(Event event) {
        super.onBrowserEvent(event);
        switch (DOM.eventGetType(event)) {
        case Event.ONMOUSEOVER:
          addStyleName("ui-state-hover");
          break;
        case Event.ONMOUSEOUT:
          removeStyleName("ui-state-hover");
          break;
        }
      }
    };
    label.setTitle(value);
    label.setWordWrap(false);
    label.setStyleName("ktr-trace-tree-item");
    DOM.setStyleAttribute(label.getElement(), "padding", "1px 3px 1px 3px");
    label.sinkEvents(Event.ONMOUSEOVER | Event.ONMOUSEOUT);

    TreeItem treeItem = new TreeItem(label){
      /** {@inheritDoc} */
      @Override
      public void setSelected(boolean selected) {
        boolean flg = (this.isSelected() == selected);
        super.setSelected(selected);
        if (!flg) {
          setStyleName(getWidget().getElement(), "ui-state-highlight", selected);
        }
      }
    };
    DOM.setStyleAttribute(treeItem.getElement(), "padding", "0");
    treeItem.setStyleName("");
    
    return treeItem;
  }
}
