/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.wjr.server.service;

import java.util.List;
import java.util.logging.Logger;

import bufferings.ktr.wjr.client.service.KtrWjrException;
import bufferings.ktr.wjr.client.service.KtrWjrService;
import bufferings.ktr.wjr.server.service.junit3.JUnit3Checker;
import bufferings.ktr.wjr.server.service.junit3.TestRunner3;
import bufferings.ktr.wjr.server.service.junit4.JUnit4Checker;
import bufferings.ktr.wjr.server.service.junit4.TestRunner4;
import bufferings.ktr.wjr.shared.model.TestCaseModel;
import bufferings.ktr.wjr.shared.model.TestStore;
import bufferings.ktr.wjr.shared.model.TestStoreConfig;

/**
 * A implementation of {@link KtrWjrService}.
 * 
 * @author bufferings[at]gmail.com
 */
public class KtrWjrServiceImpl implements KtrWjrService {

  private static final Logger logger =
    Logger.getLogger(KtrWjrServiceImpl.class.getName());

  /** {@inheritDoc} */
  public List<TestStoreConfig> loadTestStoreConfigs() {
    logger.fine("loadTestStoreConfigs");

    TestStoreConfigLoader loader = new TestStoreConfigLoader();
    return loader.loadTestStoreConfigs();
  }

  /** {@inheritDoc} */
  public TestStore loadTestStore(String storeKey) throws KtrWjrException {
    logger.fine("loadTestStore(configKey=" + storeKey + ")");

    // load config
    TestStoreConfigLoader configLoader = new TestStoreConfigLoader();
    TestStoreConfig config = configLoader.loadTestStoreConfig(storeKey);
    if (config == null) {
      throw new KtrWjrException("Configuratoin not found.");
    }

    // load store
    TestStoreLoader storeLoader = new TestStoreLoader();
    TestStore store =
      storeLoader.loadTestStore(storeKey, config.getStoreName(), config
        .getClassesDirectory(), config.getRootPackage());
    return store;
  }

  /** {@inheritDoc} */
  public List<TestCaseModel> runTests(List<TestCaseModel> testCaseModels) {
    logger.fine("runTests");

    if (JUnit4Checker.isAvailable()) {
      return new TestRunner4().runTests(testCaseModels);
    } else if (JUnit3Checker.isAvailable()) {
      return new TestRunner3().runTests(testCaseModels);
    } else {
      throw new KtrWjrException("JUnit library not found.");
    }
  }

}
