package bufferings.ktr.wjr.server.service.junit4;

import java.math.BigDecimal;
import java.util.List;
import java.util.logging.Logger;

import org.junit.runner.Request;
import org.junit.runner.Result;
import org.junit.runner.Runner;
import org.junit.runner.notification.RunListener;
import org.junit.runner.notification.RunNotifier;

import bufferings.ktr.wjr.shared.model.TestCaseModel;
import bufferings.ktr.wjr.shared.model.TestStoreItem.State;

public class TestRunner4 {

  private static final Logger logger =
    Logger.getLogger(TestRunner4.class.getName());

  /** {@inheritDoc} */
  public List<TestCaseModel> runTests(List<TestCaseModel> testCaseModels) {
    logger.fine("runTests");

    ClassLoader cl = Thread.currentThread().getContextClassLoader();

    RunNotifier notifier = new RunNotifier();
    for (TestCaseModel testCase : testCaseModels) {
      Class<?> clazz;
      try {
        clazz = Class.forName(testCase.getClassName(), false, cl);
      } catch (ClassNotFoundException e) {
        testCase.setState(State.ERROR);
        testCase.setTrace(e.getMessage());
        continue;
      }

      Request request = Request.method(clazz, testCase.getMethodName());
      Runner runner = request.getRunner();

      Result result = new Result();
      RunListener listener = result.createListener();
      notifier.addFirstListener(listener);
      notifier.fireTestRunStarted(runner.getDescription());
      try {
        long before = System.currentTimeMillis();
        runner.run(notifier);
        Long time = System.currentTimeMillis() - before;
        testCase.setTime(new BigDecimal(time.toString())
          .movePointLeft(3)
          .toPlainString());
        notifier.fireTestRunFinished(result);
      } finally {
        notifier.removeListener(listener);
      }

      if (result.getFailureCount() > 0) {
        if (result.getFailures().get(0).getException() instanceof AssertionError) {
          testCase.setState(State.FAILURE);
        } else {
          testCase.setState(State.ERROR);
        }
        testCase.setTrace(result.getFailures().get(0).getTrace());
      } else {
        testCase.setState(State.SUCCESS);
        testCase.setTrace(null);
      }
    }
    return testCaseModels;
  }

}
