/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.wjr.shared.model;

import java.io.Serializable;

/**
 * A configuration of a test store.
 * <table border=1>
 * <tr>
 * <th>Attribute</th>
 * <th>Necessary</th>
 * <th>Default</th>
 * </tr>
 * <tr>
 * <td>storeKey</td>
 * <td>Yes</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>storeName</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>maxTestCaseCountPerRequest</td>
 * <td>&nbsp;</td>
 * <td>1</td>
 * </tr>
 * <tr>
 * <td>classesDirectory</td>
 * <td>&nbsp;</td>
 * <td>WEB-INF/classes</td>
 * </tr>
 * <tr>
 * <td>rootPackage</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * </tr>
 * <tr>
 * <td>description</td>
 * <td>&nbsp;</td>
 * <td>&nbsp;</td>
 * </tr>
 * </table>
 * 
 * @author bufferings[at]gmail.com
 */
public class TestStoreConfig implements Serializable {

  private static final long serialVersionUID = 1L;

  private static final String DEFAULT_CLASSES_DIRECTORY = "WEB-INF/classes";

  private static final int DEFAULT_MAXCOUNT = 1;

  /** test store key */
  protected String storeKey;

  /** test store name */
  protected String storeName;

  /** max count of test cases run per a request */
  protected int maxTestCaseCountPerRequest = DEFAULT_MAXCOUNT;

  /** classes directory relative to web root */
  protected String classesDirectory = DEFAULT_CLASSES_DIRECTORY;

  /** root package where test cases are searched */
  protected String rootPackage = "";

  /** test store description */
  protected String description = "";

  /**
   * Default constructor for GWT-RPC serialization.
   * <p>
   * This is not used usually.
   * </p>
   */
  @Deprecated
  public TestStoreConfig() {
  }

  /**
   * Constructor
   * 
   * @param storeKey
   *          the store key
   */
  public TestStoreConfig(String storeKey) {
    this.storeKey = storeKey;
  }

  /**
   * Gets the test store key.
   * 
   * @return the test store key
   */
  public String getStoreKey() {
    return storeKey;
  }

  /**
   * Gets the test store name.
   * 
   * @return the test store name
   */
  public String getStoreName() {
    return storeName;
  }

  /**
   * Sets the test store name.
   * 
   * @param storeName
   *          the test store name
   */
  public void setStoreName(String storeName) {
    this.storeName = storeName;
  }

  /**
   * Gets the max count of test cases run per a request.
   * 
   * @return the max count of test cases run per a request
   */
  public int getMaxTestCaseCountPerRequest() {
    return maxTestCaseCountPerRequest;
  }

  /**
   * Sets the max count of test cases run per a request
   * 
   * @param maxTestCaseCountPerRequest
   *          the max count of test cases run per a request
   */
  public void setMaxTestCaseCountPerRequest(int maxTestCaseCountPerRequest) {
    this.maxTestCaseCountPerRequest = maxTestCaseCountPerRequest;
  }

  /**
   * Gets the classes directory relative to web root
   * 
   * @return the classes directory relative to web root
   */
  public String getClassesDirectory() {
    return classesDirectory;
  }

  /**
   * Sets the classes directory relative to web root
   * 
   * @param classesDirectory
   *          the classes directory relative to web root
   */
  public void setClassesDirectory(String classesDirectory) {
    this.classesDirectory = classesDirectory;
  }

  /**
   * Gets the root package where test cases are searched
   * 
   * @return the root package where test cases are searched
   */
  public String getRootPackage() {
    return rootPackage;
  }

  /**
   * Sets the root package where test cases are searched
   * 
   * @param rootPackage
   *          the root package where test cases are searched
   */
  public void setRootPackage(String rootPackage) {
    this.rootPackage = rootPackage;
  }

  /**
   * Gets the test store description
   * 
   * @return the test store description
   */
  public String getDescription() {
    return description;
  }

  /**
   * Sets the test store description
   * 
   * @param description
   *          the test store description
   */
  public void setDescription(String description) {
    this.description = description;
  }

}
