/*
 * Copyright 2009 bufferings[at]gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package bufferings.ktr.wjr.shared.util;

/**
 * Assertion utilities.
 * 
 * @author bufferings[at]gmail.com
 */
public class AssertionUtil {

  /**
   * Checks the object is not null.
   * 
   * @param obj
   *          an object to check
   * @throws NullPointerException
   *           when the object is null
   */
  public static void checkNotNull(Object obj) {
    checkNotNull(obj, null);
  }

  /**
   * Checks the object is not null.
   * 
   * @param obj
   *          an object to check
   * @param message
   *          a message to be set to the exception
   * @throws NullPointerException
   *           when the object is null
   */
  public static void checkNotNull(Object obj, String message) {
    if (obj == null) {
      throw new NullPointerException(message);
    }
  }

  /**
   * Checks the presupposition expression is fulfilled.
   * 
   * @param expression
   *          a presupposition expression
   * @throws IllegalArgumentException
   *           when the presupposition is not fulfilled
   */
  public static void checkArgument(boolean expression) {
    checkArgument(expression, null);
  }

  /**
   * Checks the presupposition expression is fulfilled.
   * 
   * @param expression
   *          a presupposition expression
   * @param message
   *          a message to be set to the exception
   * @throws IllegalArgumentException
   *           when the presupposition is not fulfilled
   */
  public static void checkArgument(boolean expression, String message) {
    if (!expression) {
      throw new IllegalArgumentException(message);
    }
  }

  /**
   * Checks the presupposition state is fulfilled.
   * 
   * @param expression
   *          a presupposition expression
   * @throws IllegalStateException
   *           when the presupposition is not fulfilled
   */
  public static void checkState(boolean expression) {
    checkState(expression, null);
  }

  /**
   * Checks the presupposition state is fulfilled.
   * 
   * @param expression
   *          a presupposition expression
   * @param message
   *          a message to be set to the exception
   * @throws IllegalStateException
   *           when the presupposition is not fulfilled
   */
  public static void checkState(boolean expression, String message) {
    if (!expression) {
      throw new IllegalStateException(message);
    }
  }
}
